/*
 * PDFlib Virtual File system (PVF):
 * Create a PVF file which holds an image or PDF, and import the data from the
 * PVF file
 *
 * This avoids disk access and is especially useful when the same image or PDF
 * is imported multiply. For examples, images which sit in a database don't
 * have to be written and re-read from disk, but can be passed to PDFlib
 * directly in memory. A similar technique can be used for loading other data
 * such as fonts, ICC profiles, etc.
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9
 */
package com.pdflib.cookbook.pdflib.general;
import java.io.*;
import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class data_from_memory
{
    public static void main (String argv[])
    {
        /* This is where the data files are. Adjust as necessary. */
        String searchpath = "../input";
        String outfile = "data_from_memory.pdf";
        String title = "PDFlib Virtual File System";

        pdflib p = null;
        int exitcode = 0;
        /* Some constant SVG data as an example for fetching input data from memory */
        String svgdata = 
        "<?xml version='1.0'?> " +
        "<svg viewBox='0 0 100 100' version='1.1' xmlns='http://www.w3.org/2000/svg'> " +
        "  <circle cx='50' cy='50' r='40' fill='orange'/> "+
        "</svg>";

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            /* Set an output path according to the name of the topic */
            if (p.begin_document(outfile, "") == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            p.create_pvf("/pvf/svg", svgdata.getBytes(), "");

            /* Load the graphics from the PVF */
            int svg = p.load_graphics("svg", "/pvf/svg" , "");
            if (svg == -1)
                throw new Exception("Error: " + p.get_errmsg());

            /* Fit the graphics on page 1 */
            p.begin_page_ext(0, 0, "width=a4.width height=a4.height");

            p.fit_graphics(svg, 350, 750, "");

            p.end_page_ext("");

            /* Fit the graphics on page 2 */
            p.begin_page_ext(0, 0, "width=a4.width height=a4.height");

            p.fit_graphics(svg, 350, 50, "");

            p.end_page_ext("");

            /* Delete the virtual file to free the allocated memory */

            p.delete_pvf("/pvf/svg");

            p.end_document("");

        } catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
            exitcode = 1;
        } catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        } finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
